import logging
import uuid
from importlib import import_module

from pyspark.sql import DataFrame
from pyspark.sql.streaming import DataStreamWriter

from pydataio.io.abstract_io import Output
from pydataio.io.abstract_io import SinkProcessor
from pydataio.io.utils.duration import parseDuration
from pydataio.io.utils.triggers import StreamingTrigger
from pydataio.io.utils.triggers import createtrigger

logger = logging.getLogger(__name__)


class SparkOutput(Output):
    """
    Class to represent a file system streaming output
    """

    format: str
    mode: str
    path: str
    partitionColumn: str
    timeout: int
    trigger: StreamingTrigger
    options: dict
    outputName: str
    sinkProcessor: SinkProcessor

    def build(self, outputConfig: dict):
        """
        Build the output
        Args:
            outputConfig: the output configuration

        """

        self.format = outputConfig["format"]
        self.mode = outputConfig["mode"]
        self.path = outputConfig["path"]
        self.timeout = parseDuration(outputConfig["timeout"])
        self.outputName = outputConfig["name"]
        self.trigger = createtrigger(outputConfig)
        self.partitionColumn = outputConfig.get("partitionColumn")
        self.options = outputConfig.get("options", {})
        self.sinkProcessor = self.createSinkTransformer(outputConfig)

        return self

    def createSinkTransformer(self, outputConfig: dict):
        """
        Create the sink transformer
        Args:
            outputConfig: the output configuration

        Returns: the sink transformer

        """
        sinkProcessorParam = outputConfig.get("sinkProcessor")
        if sinkProcessorParam is not None:
            logger.info("Using sink transformer %s", sinkProcessorParam)
            processorModulePath, processorClass = sinkProcessorParam.rsplit(".", 1)

            processorModule = import_module(processorModulePath)
            processor = getattr(processorModule, processorClass)
            return processor()
        else:
            return None

    def save(self, data: DataFrame):
        """
        Save the dataframe
        Args:
            data: the dataframe to save

        """
        logger.info("Saving dataframe to %s, format: %s, mode: %s, trigger: %s, partitionColumn: %s, options: %s", self.path, self.format, self.mode, self.trigger, self.partitionColumn, self.options)
        writer = data.writeStream.format(self.format).options(**self.options).outputMode(self.mode).queryName(self.createQueryName())
        writer = self.trigger.configureTrigger(writer)

        if self.partitionColumn is not None:
            logger.info("Partitioning by %s", self.partitionColumn)
            writer = writer.partitionBy(self.partitionColumn)

        if self.sinkProcessor is not None:
            writer = self.setupSinkProcessor(writer)

        logger.info("Saving to %s", self.path)
        streamingQuery = writer.start(self.path)
        streamingQuery.awaitTermination(timeout=self.timeout)

        streamingQuery.stop()

    def setupSinkProcessor(self, writer: DataStreamWriter):
        """
        Setup the sink processor
        Args:
            writer: the streaming writer

        Returns: the streaming writer with for each batch configured

        """
        logger.info("Setting for each batch processor %s", self.sinkProcessor)

        def writeBatch(dataFrame, batchId):
            resultDataFrame = self.sinkProcessor.process(dataFrame)
            resultDataFrame.write.format(self.format).options(**self.options).mode(self.mode).save(self.path)

        return writer.foreachBatch(writeBatch)

    def createQueryName(self):
        """
        Create a query name
        Returns: the query name

        """
        return f"QN_{self.outputName}_{uuid.uuid4()}"



from pydataio.config_handler import buildInput
from pydataio.config_handler import buildOutput
from pydataio.io.pipes.spark.batch.inputs import SparkInput as BatchInput
from pydataio.io.pipes.spark.batch.outputs import SparkOutput as BatchOutput
from pydataio.io.pipes.spark.streaming.inputs import SparkInput as StreamingInput
from pydataio.io.pipes.spark.streaming.outputs import SparkOutput as StreamingOutput
from pydataio.io.utils.triggers import AvailableNowTrigger
from pydataio.schema_registry import SchemaRegistry
from tests.dummies import dummyEventSchema


def test_buildInput_single_batch():
    conf: list[dict] = [
        {
            "name": "events",
            "type": "pipes.spark.batch.SparkInput",
            "format": "json",
            "path": "tests/resources/data/events",
            "schema": "dummyEventSchema",
            "options": {"header": "true"},
        }
    ]

    registry = SchemaRegistry().registerSchema("dummyEventSchema", dummyEventSchema)
    reader = buildInput(conf, registry)

    assert isinstance(reader["events"], BatchInput)
    assert reader["events"].format == "json"
    assert reader["events"].path == "tests/resources/data/events"
    assert reader["events"].schema == dummyEventSchema
    assert reader["events"].options == {"header": "true"}


def test_buildInput_single_streaming():
    conf: list[dict] = [
        {
            "name": "events",
            "type": "pipes.spark.streaming.SparkInput",
            "format": "json",
            "path": "tests/data/streaming",
            "schema": "dummyEventSchema",
            "options": {"header": "true"},
        }
    ]

    registry = SchemaRegistry().registerSchema("dummyEventSchema", dummyEventSchema)
    reader = buildInput(conf, registry)

    assert isinstance(reader["events"], StreamingInput)
    assert reader["events"].format == "json"
    assert reader["events"].path == "tests/data/streaming"
    assert reader["events"].schema == dummyEventSchema
    assert reader["events"].options == {"header": "true"}


def test_buildInput_mix():
    conf: list[dict] = [
        {
            "name": "events",
            "type": "pipes.spark.batch.SparkInput",
            "format": "json",
            "path": "tests/resources/data/events",
            "schema": "dummyEventSchema",
            "options": {"header": "true"},
        },
        {
            "name": "rules",
            "type": "pipes.spark.streaming.SparkInput",
            "format": "parquet",
            "path": "tests/data/rules",
            "schema": "dummyEventSchema",
            "options": {"header": "true"},
        },
    ]

    registry = SchemaRegistry().registerSchema("dummyEventSchema", dummyEventSchema)
    reader = buildInput(conf, registry)

    assert isinstance(reader["events"], BatchInput)
    assert reader["events"].format == "json"
    assert reader["events"].path == "tests/resources/data/events"
    assert reader["events"].schema == dummyEventSchema
    assert reader["events"].options == {"header": "true"}

    assert isinstance(reader["rules"], StreamingInput)
    assert reader["rules"].format == "parquet"
    assert reader["rules"].path == "tests/data/rules"
    assert reader["rules"].schema == dummyEventSchema
    assert reader["rules"].options == {"header": "true"}


def test_buildInput_mix_not_optional():
    conf: list[dict] = [
        {
            "name": "events",
            "type": "pipes.spark.batch.SparkInput",
            "format": "json",
            "path": "tests/resources/data/events",
        },
        {
            "name": "rules",
            "type": "pipes.spark.streaming.SparkInput",
            "format": "parquet",
            "path": "tests/data/rules",
        },
    ]

    registry = SchemaRegistry().registerSchema("selAggregatedEvent", dummyEventSchema)
    reader = buildInput(conf, registry)

    assert isinstance(reader["events"], BatchInput)
    assert reader["events"].format == "json"
    assert reader["events"].path == "tests/resources/data/events"
    assert reader["events"].schema is None
    assert reader["events"].options == {}

    assert isinstance(reader["rules"], StreamingInput)
    assert reader["rules"].format == "parquet"
    assert reader["rules"].path == "tests/data/rules"
    assert reader["rules"].schema is None
    assert reader["rules"].options == {}


def test_buildInput_none():
    conf = None

    registry = SchemaRegistry().registerSchema("selAggregatedEvent", dummyEventSchema)
    reader = buildInput(conf, registry)

    assert reader is None


def test_buildSparkOutput_batch():
    conf: dict = {
        "type": "pipes.spark.batch.SparkOutput",
        "format": "json",
        "path": "tests/data/output",
        "mode": "overwrite",
        "partitionColumn": "date",
        "options": {"header": "true"},
    }

    writer = buildOutput(conf)

    assert isinstance(writer, BatchOutput)
    assert writer.format == "json"
    assert writer.path == "tests/data/output"
    assert writer.mode == "overwrite"
    assert writer.partitionColumn == "date"
    assert writer.options == {"header": "true"}


def test_buildSparkOutput_batch_not_optional():
    conf: dict = {
        "type": "pipes.spark.batch.SparkOutput",
        "format": "json",
        "path": "tests/data/output",
        "mode": "overwrite",
    }

    writer = buildOutput(conf)

    assert isinstance(writer, BatchOutput)
    assert writer.format == "json"
    assert writer.path == "tests/data/output"
    assert writer.mode == "overwrite"
    assert writer.partitionColumn is None
    assert writer.options == {}


def test_buildSparkOutput_streaming():
    conf: dict = {
        "name": "events",
        "type": "pipes.spark.streaming.SparkOutput",
        "format": "json",
        "path": "tests/data/output",
        "mode": "overwrite",
        "partitionColumn": "date",
        "sinkProcessor": "tests.dummies.DummyProcessor",
        "timeout": "2 minutes",
        "trigger": "AvailableNowTrigger",
        "options": {"header": "true"},
    }

    writer = buildOutput(conf)

    assert isinstance(writer, StreamingOutput)
    assert writer.outputName == "events"
    assert writer.format == "json"
    assert writer.path == "tests/data/output"
    assert writer.mode == "overwrite"
    assert writer.partitionColumn == "date"
    assert writer.timeout == 120
    assert isinstance(writer.trigger, AvailableNowTrigger)
    assert writer.options == {"header": "true"}


def test_buildSparkOutput_steaming_not_optional():
    conf: dict = {
        "name": "events",
        "type": "pipes.spark.streaming.SparkOutput",
        "format": "json",
        "path": "tests/data/output",
        "mode": "overwrite",
        "timeout": "2 minutes",
        "trigger": "AvailableNowTrigger",
    }

    writer = buildOutput(conf)

    assert isinstance(writer, StreamingOutput)
    assert writer.outputName == "events"
    assert writer.format == "json"
    assert writer.path == "tests/data/output"
    assert writer.mode == "overwrite"
    assert writer.partitionColumn is None
    assert writer.timeout == 120
    assert isinstance(writer.trigger, AvailableNowTrigger)
    assert writer.options == {}
    assert writer.sinkProcessor is None


def test_buildSparkOutput_none():
    conf = None

    writer = buildOutput(conf)

    assert writer is None



from importlib import import_module

from pydataio.io import IO_BASENAME


def getClass(identifier, package):
    """
    Dynamically imports and returns a class from a specified module.

    Args:
        identifier (str): The identifier for the class, in the format 'batch.FileSystemInput'.
        package (str, optional): The sub package, i.e.  "outputs".

    Returns:
        type: The class specified by the pipe_identifier.

    Raises:
        ValueError: If the pipe_identifier does not contain a dot separating the module and class name.
    """
    if "." in identifier:
        module, className = identifier.rsplit(".", 1)
        mod = import_module(f"{IO_BASENAME}.{module}.{package}")
        classType = getattr(mod, className)
    else:
        raise ValueError(f"Invalid pipe identifier: {identifier}")
    return classType



import logging

from azure.identity import ClientSecretCredential

from .config_handler import loadConfiguration
from .schema_registry import SchemaRegistry

logger = logging.getLogger(__name__)


class Pipeline:
    """
    Pipeline class to run the data processing pipeline
    """

    schemaRegistry: SchemaRegistry

    def __init__(self):
        self.schemaRegistry = SchemaRegistry()

    def run(self, confPath: str, credential: ClientSecretCredential = None, additionalArgs: dict = None):
        """
        Run the data processing pipeline
        Args:
            confPath: path to the configuration file
        """
        logger.info("Loading configuration from %s", confPath)
        config = loadConfiguration(confPath, self.schemaRegistry, credential)
        config.transformer.run(config.JobConfig, additionalArgs)



from datetime import timedelta
import re


def parseDuration(duration: str):
    """
    Parse a duration string
    Args:
        duration: the duration string

    Returns: the duration in seconds

    """
    pattern = re.compile(r"(\d+)\s*(seconds?|minutes?|hours?|days?)")
    match = pattern.match(duration)
    if not match:
        raise ValueError(f"Invalid duration string: {duration}")

    value, unit = match.groups()
    value = int(value)
    unit = unit.lower()

    if unit.startswith("second"):
        duration = timedelta(seconds=value)
    elif unit.startswith("minute"):
        duration = timedelta(minutes=value)
    elif unit.startswith("hour"):
        duration = timedelta(hours=value)
    elif unit.startswith("day"):
        duration = timedelta(days=value)
    else:
        raise ValueError(f"Unknown time unit: {unit}")

    return duration.total_seconds()



import logging
import os
import shutil

import yaml


def cleanCheckpoint(configPath: str):
    """
    Clean the checkpoint directory
    Args:
        configPath: path to the yaml configuration file

    """
    with open(configPath) as file:
        config = yaml.safe_load(file)

    checkpointPath = config["Output"]["options"].get("checkpointLocation", "")
    if os.path.exists(checkpointPath):
        shutil.rmtree(checkpointPath)
        logging.info("Checkpoint directory %s cleaned", checkpointPath)
    else:
        logging.info("Checkpoint directory %s does not exist", checkpointPath)



import logging

from pyspark.sql import DataFrame, SparkSession
from pyspark.sql.types import (
    StructType,
    StructField,
    StringType,
)

from pydataio.io.abstract_io import SinkProcessor
from pydataio.job_config import JobConfig
from pydataio.transformer import Transformer

logger = logging.getLogger(__name__)


class DummyProcessor(SinkProcessor):
    """
    Dummy processor for test purpose
    """

    def process(self, data: DataFrame):
        logger.info("Processing data with %s rows", data.count())
        return data


class DummyTransformer(Transformer):
    """
    Dummy transformer for test purpose
    """

    def featurize(self, jobConfig: JobConfig, spark: SparkSession, additionalArgs: dict = None):
        events = jobConfig.load(inputName="events", spark=spark)

        events.groupBy("id").count()

        jobConfig.writer.save(data=events)


class DummyTransformerNoIO(Transformer):
    """
    Dummy transformer without input output for test purpose
    """

    def featurize(self, jobConfig: JobConfig, spark: SparkSession, additionalArgs: dict = None):
        logger.info("Featurizing data without IO from %s", jobConfig.parameters["dummyParam"])


dummyEventSchema = StructType(
    [
        StructField("id", StringType(), False),
        StructField("phase", StringType(), True),
        StructField("value", StringType(), True),
    ]
)



import logging
from logging.config import fileConfig
import os

# Path to the logging configuration file
config_path = os.path.join(os.path.dirname(__file__), "config", "logging.ini")

logging.info("Config path: %s", config_path)

# Load the logging configuration
fileConfig(config_path)



import logging

from pyspark.sql import DataFrame

from pydataio.io.abstract_io import Output

logger = logging.getLogger(__name__)


class SparkOutput(Output):
    """
    Class to represent a file system output
    """

    format: str
    mode: str
    path: str
    partitionColumn: str
    options: dict

    def build(self, outputConfig: dict):
        """
        Build the output
        Args:
            outputConfig: the output configuration

        """
        self.format = outputConfig["format"]
        self.mode = outputConfig["mode"]
        self.path = outputConfig["path"]
        self.partitionColumn = outputConfig.get("partitionColumn")
        self.options = outputConfig.get("options", {})

        return self

    def save(self, data: DataFrame):
        """
        Save the dataframe
        Args:
            data: the dataframe to save

        """

        logger.info("Saving dataframe to %s, format: $s, mode: %s, partitionColumn: %s, options: %s", self.path, self.format, self.mode, self.partitionColumn, self.options)
        writer = data.write.format(self.format).options(**self.options).mode(self.mode)

        if self.partitionColumn is not None:
            logger.info("Partitioning by %s", self.partitionColumn)
            writer = writer.partitionBy(self.partitionColumn)

        logger.info("Saving to %s", self.path)
        return writer.save(self.path)



from pyspark.sql import SparkSession
from pyspark.sql.types import StructType

from pydataio.io.abstract_io import Input
from pyspark.sql.dataframe import DataFrame
from pyspark.sql.streaming.readwriter import DataStreamReader


class SparkInput(Input):
    """
    Class to represent a file system streaming input
    """

    format: str
    path: str
    options: dict
    schema: StructType

    def __init__(self, format: str, path: str, options: dict = None, schema: StructType = None):
        if options is None:
            options = {}

        self.format = format
        self.options = options
        self.schema = schema
        self.path = path

    def load(self, spark: SparkSession) -> DataFrame:
        """
        Load a dataframe
        Args:
            spark: the spark session

        Returns: the dataframe

        """
        reader: DataStreamReader = spark.readStream.format(self.format).options(**self.options)

        if self.schema is not None:
            reader = reader.schema(self.schema)

        return reader.load(self.path)



from typing import Optional

from pyspark.sql import SparkSession
from pyspark.sql.dataframe import DataFrame

from pydataio.io.pipes.spark.batch.inputs import Input
from pydataio.io.pipes.spark.batch.outputs import Output


class JobConfig:
    """
    Class to represent a job configuration
    """

    name: str
    parameters: dict
    reader: Optional[dict[str, Input]]
    writer: Optional[Output]

    def __init__(
        self,
        name: str,
        reader: dict[str, Input],
        writer: Output,
        parameters: dict = None,
    ):
        if parameters is None:
            parameters = {}

        self.name = name
        self.parameters = parameters
        self.reader = reader
        self.writer = writer

    def load(self, inputName: str, spark: SparkSession) -> DataFrame:
        """
        Load a dataframe from an input

        Args:
            inputName: name of the input
            spark: the spark session

        Returns: the dataframe

        """
        if self.reader is None:
            raise ValueError("No reader defined in the configuration")

        return self.reader[inputName].load(spark)



from abc import ABC, abstractmethod

from pyspark.sql import DataFrame
from pyspark.sql import SparkSession


class SinkProcessor(ABC):
    """
    Abstract class to transform data in a forEachBatch context
    """

    @abstractmethod
    def process(self, data: DataFrame):
        pass


class Input(ABC):
    """
    Abstract class to represent an input
    """

    @abstractmethod
    def load(self, spark: SparkSession) -> DataFrame:
        pass


class Output(ABC):
    """
    Abstract class to represent an output
    """

    @abstractmethod
    def build(self, outputConfig: dict):
        pass

    @abstractmethod
    def save(self, data: DataFrame):
        pass



from abc import ABC, abstractmethod

from pyspark.sql.streaming import DataStreamWriter

from pydataio.io.utils.class_helper import getClass


class StreamingTrigger(ABC):
    """
    Abstract class for a streaming trigger
    """

    @abstractmethod
    def configureTrigger(self, writer: DataStreamWriter):
        pass


class ProcessingTimeTrigger(StreamingTrigger):
    """
    Class to represent a processing time trigger
    """

    interval: str

    def __init__(self, interval: str):
        self.interval = interval

    def configureTrigger(self, writer: DataStreamWriter):
        return writer.trigger(processingTime=self.interval)


class OneTimeTrigger(StreamingTrigger):
    """
    Class to represent a one time trigger
    """

    def configureTrigger(self, writer: DataStreamWriter):
        return writer.trigger(once=True)


class AvailableNowTrigger(StreamingTrigger):
    """
    Class to represent an available now trigger
    """

    def configureTrigger(self, writer: DataStreamWriter):
        return writer.trigger(availableNow=True)


def createtrigger(outputConfig: dict):
    """
    Create the streaming trigger

    Args:
        TriggerClass: the trigger class
        outputConfig: the output configuration

    Returns: the streaming trigger

    """
    triggerType = outputConfig["trigger"]
    TriggerClass = getClass(f"utils.{triggerType}", "triggers")

    if triggerType == "ProcessingTimeTrigger" or triggerType == "ContinuousTrigger":
        return TriggerClass(outputConfig["duration"])
    else:
        return TriggerClass()



from pydataio.io.utils.duration import parseDuration


def test_parseDuration_hours():
    assert parseDuration("1 hour") == 3600


def test_parseDuration_minutes():
    assert parseDuration("1 minute") == 60


def test_parseDuration_seconds():
    assert parseDuration("1 second") == 1



from pyspark.sql.types import StructType


class SchemaRegistry:
    """
    Class to represent a schema registry
    """

    def __init__(self):
        self.schemas = {}

    def get_schema(self, shemaName: str):
        """
        Get a schema from the registry
        Args:
            shemaName: name of the schema

        Returns: the schema

        """
        return self.schemas.get(shemaName, None)

    def registerSchema(self, schemaName: str, schema: StructType):
        """
        Register a schema in the registry

        Args:
            schemaName: name of the schema
            schema:  the schema to register

        """
        self.schemas[schemaName] = schema
        return self



import logging
from importlib import import_module
from typing import Optional
from urllib.parse import urlparse

import fsspec
import yaml
from azure.identity import ClientSecretCredential

from pydataio.io.pipes.spark.batch.inputs import Input
from pydataio.io.pipes.spark.batch.outputs import Output
from .io.utils.class_helper import getClass
from .job_config import JobConfig
from .pipeline_config import PipelineConfig
from .schema_registry import SchemaRegistry

logger = logging.getLogger(__name__)


def loadConfiguration(configPath: str, schemaRegistry: SchemaRegistry, credential: ClientSecretCredential = None):
    """
    Load a yaml configuration file from a path
    Args:
        configPath: path to the yaml configuration file

    Returns: JobConfig object

    """
    config = parse_config(configPath, credential)

    transformerModulePath, transformerClass = config["Processing"]["type"].rsplit(".", 1)
    transformerModule = import_module(transformerModulePath)
    transformer = getattr(transformerModule, transformerClass)

    reader = buildInput(config.get("Input", None), schemaRegistry)
    writer = buildOutput(config.get("Output", None))

    jobConfig = JobConfig(
        name=transformerClass,
        reader=reader,
        writer=writer,
        parameters=config["Processing"].get("parameters", None),
    )
    return PipelineConfig(transformer=transformer(), JobConfig=jobConfig)


def parse_config(configPath: str, credential: ClientSecretCredential = None):
    """
    Load and parse a YAML configuration file from a given path.

    Args:
        configPath (str): Path to the YAML configuration file.
        credential (ClientSecretCredential, optional): Credential for accessing Azure Blob Storage.

    Returns:
        dict: Parsed configuration as a dictionary.
    """
    logger.info("Loading configuration from %s", configPath)

    parsed_url = urlparse(configPath)
    if parsed_url.scheme == "abfss":
        dfs_core_windows_net = ".dfs.core.windows.net"
        assert dfs_core_windows_net in parsed_url.hostname
        storage_account_name = parsed_url.hostname.split(dfs_core_windows_net)[0]

        fsspec_handle = fsspec.open(
            configPath,
            mode="rt",
            account_name=storage_account_name,
            credential=credential,
        )
    else:
        fsspec_handle = fsspec.open(configPath, mode="rt")

    with fsspec_handle.open() as file:
        config = yaml.safe_load(file)
    logger.info("Configuration parsed: %s", config)
    return config


def buildInput(inputConfig: Optional[list[dict]], schemaRegistry: SchemaRegistry) -> Optional[dict[str, Input]]:
    """
    Build an Input object from a dictionary
    Args:
        inputConfig: dictionary containing the configuration for the FileSystemInput object

    Returns: the FileSystemInput object

    """
    if inputConfig is None:
        logger.warning("No input configuration found!")
        return None

    inputs = {}

    for input in inputConfig:
        ReaderClass = getClass(input["type"], "inputs")
        inputs[input["name"]] = ReaderClass(
            format=input["format"],
            path=input["path"],
            options=input.get("options", None),
            schema=schemaRegistry.get_schema(input.get("schema", "")),
        )

    return inputs


def buildOutput(outputConfig: Optional[dict]) -> Optional[Output]:
    """
    Build an Output object from a dictionary
    Args:
        outputConfig: dictionary containing the configuration for the FileSystemOutput object

    Returns: the FileSystemOutput object

    """
    if outputConfig is None:
        logger.warning("No output configuration found!")
        return None

    WriterClass = getClass(outputConfig["type"], "outputs")

    return WriterClass().build(outputConfig)



from .job_config import JobConfig
from .transformer import Transformer


class PipelineConfig:
    """
    Class to represent a pipeline configuration
    """

    transformer: Transformer
    JobConfig: JobConfig

    def __init__(self, transformer: Transformer, JobConfig: JobConfig):
        self.transformer = transformer
        self.JobConfig = JobConfig



import logging
from abc import ABC, abstractmethod

from pyspark.sql import SparkSession

from .job_config import JobConfig

logger = logging.getLogger(__name__)


class Transformer(ABC):
    """
    Abstract class for a data transformer
    """

    @abstractmethod
    def featurize(self, jobConfig: JobConfig, spark: SparkSession, additionalArgs: dict = None):
        pass

    def run(self, jobConfig: JobConfig, additionalArgs: dict = None):
        """
        Run the transformer
        Args:
            jobConfig: the job configuration
        """
        logger.info("Running transformer %s", jobConfig.name)
        logger.info("Getting spark session")
        sparkSession = SparkSession.builder.appName(jobConfig.name).getOrCreate()
        try:
            logger.info("Running transformer")
            self.featurize(jobConfig, sparkSession, additionalArgs)
        except Exception as e:
            logger.error("Error running transformer %s", e)
            raise e

        logger.info("End of the pipeline")



import pydataio


def test_createProcessingTimeTrigger():
    conf = {"trigger": "ProcessingTimeTrigger", "duration": "10 seconds"}
    trigger = pydataio.io.utils.triggers.createtrigger(conf)

    assert isinstance(trigger, pydataio.io.utils.triggers.ProcessingTimeTrigger)
    assert trigger.interval == "10 seconds"


def test_createOneTimeTrigger():
    conf = {"trigger": "OneTimeTrigger"}
    trigger = pydataio.io.utils.triggers.createtrigger(conf)

    assert isinstance(trigger, pydataio.io.utils.triggers.OneTimeTrigger)


def test_createAvailableNowTrigger():
    conf = {"trigger": "AvailableNowTrigger"}
    trigger = pydataio.io.utils.triggers.createtrigger(conf)

    assert isinstance(trigger, pydataio.io.utils.triggers.AvailableNowTrigger)


def test_createContinuousTrigger():
    conf = {"trigger": "ContinuousTrigger", "duration": "10 seconds"}

    try:
        pydataio.io.utils.triggers.createtrigger(conf)
        raise AssertionError("Should raise an exception")
    except AttributeError:
        assert True



import logging
import os

from pyspark.context import SparkContext

from pydataio.pipeline import Pipeline
from tests import root_tests

from . import test_utils
from .dummies import dummyEventSchema

logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(os.environ.get("LOG_LEVEL", logging.INFO))


def test_transformer_batch():
    confPath = os.path.join(root_tests, "resources/conf/testConfig.yaml")
    sc = SparkContext.getOrCreate()
    sc.setLogLevel("INFO")
    pipeline = Pipeline()
    pipeline.schemaRegistry.registerSchema("dummyEvent", dummyEventSchema)
    pipeline.run(confPath)

    assert True


def test_transformer_streaming():
    confPath = os.path.join(root_tests, "resources/conf/streamingTestConfig.yaml")
    sc = SparkContext.getOrCreate()
    sc.setLogLevel("INFO")
    pipeline = Pipeline()
    pipeline.schemaRegistry.registerSchema("dummyEvent", dummyEventSchema)
    pipeline.run(confPath)

    test_utils.cleanCheckpoint(confPath)

    assert True


def test_transformer_no_io():
    confPath = os.path.join(root_tests, "resources/conf/testConfigNoIO.yaml")
    sc = SparkContext.getOrCreate()
    sc.setLogLevel("INFO")
    pipeline = Pipeline()
    pipeline.run(confPath)

    assert True


def test_transformer_no_io_add_params():
    confPath = os.path.join(root_tests, "resources/conf/testConfigNoIO.yaml")
    sc = SparkContext.getOrCreate()
    sc.setLogLevel("INFO")
    pipeline = Pipeline()
    pipeline.run(confPath, additionalArgs={"dummyParam": "dummyValue"})

    assert True
